/**
 * Class: OpenLayers.Format.SOS
 * Superclass for SOS parser
 *
 * Inherits from:
 *  - <OpenLayers.Format.XML>
 *
 * @date: 10. August 2009
 * @author: Dominik Helle, Till Adams
 * @version: 0.6
 *
 */
/**

OpenLayers.Format.SOS = OpenLayers.Class(OpenLayers.Format.XML, {

    /**
     * Property: NAMESPACES
     * {Object} Mapping of namespace aliases to namespace URIs.
     */
    NAMESPACES: {
	  	ows: "http://www.opengis.net/ows", //xmlns:ows
	  	gml: "http://www.opengis.net/gml", //xmlns:ows
	  	sos: "http://www.opengis.net/sos/1.0", //xmlns:sos
	  	xmlns: "http://www.opengis.net/sos/1.0", //xmlns
	  	xsi: "http://www.w3.org/2001/XMLSchema-instance", //xmlns
	  	ogc: "http://www.opengis.net/ogc",
	  	schemaLocation: "http://www.opengis.net/sos/1.0 http://schemas.opengis.net/sos/1.0.0/sosGetCapabilities.xsd", //xsi:schemaLocation
	    DSschemaLocation: "http://www.opengis.net/sos/1.0 http://schemas.opengis.net/sos/1.0.0/sosDescribeSensor.xsd" //Describe Sensor
    },

   /**
    *
    */
 	SERVICE: "SOS",

   /**
    *
    */
	VERSION: "1.0.0",

   /**
    *
    */
	UPDATESEQUENCE: "12",

	gmlNS: "http://www.opengis.net/gml",
   /**
    *
    */
	OUTPUTFORMAT_SML: "text/xml; subtype=sensorML/1.0.0",

   /**
    *
    */
  OUTPUTFORMAT_OM:   "text/xml; subtype=om/1.0.0",

   /**
    *
    */
    initialize: function(options) {
  			OpenLayers.Format.XML.prototype.initialize.apply(this, [options]);
    },


   /**
    * create a getCapabilites XML
    */
    write_GetCapabilities: function() {
    var root = this.createElementDefaultNS("sos:GetCapabilities");

		this.setAttributes(root, {
		 		    'service':   this.SERVICE,
		   	   'updateSequence': this.UPDATESEQUENCE
		 });

			// node, uri, name, value
		  root = this.setSOSNamespaces(root);

		 	 root.appendChild(this.createElementDefaultNS("ows:AcceptVersions") );
			root.firstChild.appendChild(this.createElementDefaultNS("ows:Version", this.VERSION));
       return OpenLayers.Format.XML.prototype.write.apply(this, [root]);
    },

	/**
	 *
	 */
    write_DescribeSensor: function(id) {

    var root = this.createElementDefaultNS("sos:DescribeSensor");
		var procedure = "urn:ogc:def:procedure:";

		this.setAttributes(root, {
 		    'service': this.SERVICE,
			  'version': this.VERSION,
			  'outputFormat': this.OUTPUTFORMAT_SML
     });

		this.setDSNamespaces(root);
		// put procedure together
		procedure = procedure + id;

 	  root.appendChild(this.createElementNS(this.NAMESPACES.sos, "procedure"));
    root.firstChild.appendChild(this.createTextNode(procedure));

    return OpenLayers.Format.XML.prototype.write.apply(this, [root]);
    },

	/*
	 *
	 */
	write_GetObservation: function(options) {
	    var root = this.createElementDefaultNS("sos:GetObservation");
			this.setAttributes(root, {
	 		    'service': this.SERVICE,
			   	'version': this.VERSION
       });


		root = this.setSOSNamespaces(root);
  	root.appendChild(this.createElementDefaultNS("sos:responseFormat", this.OUTPUTFORMAT_OM ));

		for (var key in options) {

			 if (key != "time") {
				root.appendChild(this.createElementDefaultNS("sos:"+key, options[key]));
			 	} else {
			 		var eventTime = this.createEventTime(options[key]);
			    		root.appendChild(eventTime);

			 	}
		}
			return OpenLayers.Format.XML.prototype.write.apply(this, [root]);
	},




	/**
	 *
	 *
	 */
	createEventTime: function(options) {
		 var eventTime = this.createElementDefaultNS("sos:eventTime");
		 eventTime.appendChild(this.createElementNS(this.NAMESPACES.ogc,"ogc:TM_During"));
		 eventTime.firstChild.appendChild(this.createElementNS(this.NAMESPACES.ogc,"ogc:PropertyName"));
		 var time = options.split("|");

		 var timePeriod = this.createElementNS(this.NAMESPACES.gml,"gml:TimePeriod");
		 timePeriod.appendChild(this.createElementNS(this.NAMESPACES.gml,"beginPosition"));
		 timePeriod.appendChild(this.createElementNS(this.NAMESPACES.gml,"endPosition"));
	   timePeriod.firstChild.appendChild(this.createTextNode(time[0]));
	   timePeriod.lastChild.appendChild(this.createTextNode(time[1]));

	 	 eventTime.firstChild.appendChild(timePeriod);

	   return eventTime;
	},

	/**
	 *
	 *
	 */

	setDSNamespaces: function(node) {
	  // node, uri, name, value
	 	 this.setAttributeNS(
		   	node, this.NAMESPACES.xsi, 'xsi:schemaLocation', this.NAMESPACES.DSschemaLocation
   	);
	},


	setSOSNamespaces: function(node) {
	    // node, uri, name, value
		this.setAttributeNS(
		 	node, this.NAMESPACES.xsi, 'xsi:schemaLocation', this.NAMESPACES.schemaLocation
		);

		this.setAttributeNS(
			node, this.NAMESPACES.ogc, 'ogc:ogc', this.NAMESPACES.ogc
		);

		this.setAttributeNS(
			node, this.NAMESPACES.gml, 'gml:gml', this.NAMESPACES.gml
		);


		return node;
	},

	/**
	 *
	 *
	 */
	createElementDefaultNS: function(name, childValue, attributes) {
        var node = this.createElementNS(
            this.NAMESPACES.sos,
            name
        );
        if(childValue) {
            node.appendChild(this.createTextNode(childValue));
        }
        if(attributes) {
            this.setAttributes(node, attributes);
        }
        return node;
    },

	/**
	 *
	 *
	 */
	setAttributes: function(node, obj) {
        var value;
        for(var name in obj) {
            value = obj[name].toString();

            if(value.match(/[A-Z]/)) {
                this.setAttributeNS(node, null, name, value);
            } else {
                node.setAttribute(name, value);
            }
        }
    },

	/**
	 *
	 *
	 */
 	read_LatLon: function(xmlNode){

        var point = this.getChildValue(xmlNode, 'http://www.opengis.net/gml', "pos");
	var floats = point.split(" ");

        return new OpenLayers.LonLat(parseFloat(floats[0]),
 	parseFloat(floats[1]) );
    },


		/**
		 *
		 *
		 */
 	getChildValue: function(node, nsuri, name, def) {
		        var value;
			        var eles = this.getElementsByTagNameNS(node, nsuri, name);
			        if(eles && eles[0] && eles[0].firstChild
			            && eles[0].firstChild.nodeValue) {
			            value = eles[0].firstChild.nodeValue;
			        } else {
				  value = (def == undefined) ? "" : def;
			        }
			  return value;
		},



	 getElementsByTagNameNS: function(node, uri, name) {
	        var elements = [];
	        if(node.getElementsByTagNameNS) {
	            elements = node.getElementsByTagNameNS(uri, name);
	        } else {
	            // brute force method
	            var allNodes = node.getElementsByTagName("*");
	            var potentialNode, fullName;
	            for(var i=0; i<allNodes.length; ++i) {
	                potentialNode = allNodes[i];
	                fullName = (potentialNode.prefix) ?
	                           (potentialNode.prefix + ":" + name) : name;
	                if((name == "*") || (fullName == potentialNode.nodeName)) {
	                    if((uri == "*") || (uri == potentialNode.namespaceURI)) {
	                        elements.push(potentialNode);
	                    }
	                }
	            }
	        }
	        return elements;
	    },


		/**
		 *
		 *
		 */
	read_TimeStamp: function(xmlNode, i) {
		var timeElements = this.getElementsByTagNameNS(xmlNode, this.gmlNS, "timePosition");
		var time = [];

		for (var i = 0; i < timeElements.length; i++) {
		   time[i] = timeElements[i].firstChild.nodeValue;
		 }

		 return time;
	 },

	/**
	 *
	 *
	 */
	read_ObserverdProperty: function(xmlNode) {
		 var observerdPropertieElements = this.getElementsByTagNameNS(xmlNode, "", "observedProperty");
		 var observerdProperties = [];

		 for (var i = 0; i < observerdPropertieElements.length; i++) {
		    observerdProperties[i] = observerdPropertieElements[i].firstChild.nodeValue;
		  }

		 return observerdProperties;
	},

	/**
	 *
	 *
	 */
	read_Value: function(xmlNode) {
		 var valueElements = this.getElementsByTagNameNS(xmlNode, "", "temperature");
		 var value = [];

		 for (var i = 0; i < valueElements.length; i++) {
		    value[i] = valueElements[i].firstChild.nodeValue;
		  }

			return value;
	},

    CLASS_NAME: "OpenLayers.Format.SOS"
});